
-- local upvalues
local select = select
local UnitClass = UnitClass
local UnitIsDead = UnitIsDead
local UnitIsGhost = UnitIsGhost
local UnitIsPlayer = UnitIsPlayer
local UnitReaction = UnitReaction
local UnitIsConnected = UnitIsConnected

local oUF = _G["oUF"] 

if not oUF then
	oUF = oUFembed
	oUFembed = nil
end

local print = function(a) ChatFrame1:AddMessage("|cff33ff99oUF:|r "..tostring(a)) end

local playerClass = select(2, UnitClass("player")) 

local font, fontHeight = "Interface\\AddOns\\oUF_sb\\media\\barframes.ttf", 11
--local font, fontHeight = "Interface\\AddOns\\oUF_sb\\media\\Russel Square LT.ttf", 9

local backdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		insets = {left = -1.5, right = -1.5, top = -1.5, bottom = -1.5},
	}
local statusbartexture = "Interface\\AddOns\\oUF_sb\\media\\statusbar"
local bordertexture = "Interface\\AddOns\\oUF_sb\\media\\border"
local druidmanabartexture = "Interface\\AddOns\\oUF_sb\\media\\statusbar"

local LibDruidMana
if LibStub then LibDruidMana = LibStub("LibDruidMana-1.0", true) end	


local barFormatMinMax = "%d/%d"
local barFormatPerc = "%d%%"
local barFormatPercMinMax = barFormatPerc.." "..barFormatMinMax
local barFormatDeficit = "|cffff8080%d|r"
local barFormatMinMaxDef = "%d/%d |cffff8080%d|r"

local function fmt_standard(txt, min, max)
	txt:SetFormattedText(barFormatMinMax, min, max)
end

local function fmt_perc(txt, min, max)
	local perc = floor(min/max*100)
	txt:SetFormattedText(barFormatPerc,perc)
end

local function fmt_full(txt, min, max)
	local deficit = min - max
	if deficit == 0 then
		txt:SetFormattedText(barFormatMinMax, min, max )
	else
		txt:SetFormattedText(barFormatMinMaxDef, min, max, deficit)
	end
end

local function fmt_deficit(txt, min, max)
	local deficit = min - max
	if deficit < 0 then
		txt:SetFormattedText(barFormatDeficit, deficit)
	else
		txt:SetText(max)
	end
end

local function fmt_percminmax(txt, min, max)
	local perc = floor(min/max*100)
	txt:SetFormattedText(barFormatPercMinMax, perc, min, max)
end

local fmtmeta = { __index = 
	function(self, key)
		if type(key) == "nil" then return nil end
		if not rawget(self, key) then
			rawset(self, key, fmt_standard)
			return self[key]
		end
	end
}

local formats = setmetatable({}, { __index = 
	function(self, key)
		if type(key) == "nil" then return nil end
		if not rawget(self, key) then
			if key:find("raidpet%d") then self[key] = self.raidpet
			elseif key:find("raidtarget%d") then self[key] = self.raidtarget
			elseif key:find("raid%d") then self[key] = self.raid
			elseif key:find("party6%d") then self[key] = self.partypet
			elseif key:find("party%dtarget") then self[key] = self.partytarget
			elseif key:find("party%d") then self[key] = self.party
			else
				self[key] = {}
			end
		end
		return self[key]
	end,
	__newindex = function(self, key, value)
		rawset(self, key, setmetatable(value, fmtmeta))
	end,
})


local UnitSettings = setmetatable({
	player = {
		["initial-width"] = 200,
		["initial-height"] = 39,
		["heightHealth"] = 22,
		["heightPower"] = 16,
		powertext = true,
		showbuffs = true,
	},
	target = {
		["initial-width"] = 200,
		["initial-height"] = 39,
		["heightHealth"] = 22,
		["heightPower"] = 16,
		powertext = true,
		showbuffs = true,
	},
	party = {
		["initial-width"] = 200,
		["initial-height"] = 39,
		["heightHealth"] = 22,
		["heightPower"] = 16,
		powertext = true,
		showbuffs = true,
	},	
	targettarget = {
		["initial-width"] = 175,
		["initial-height"] = 20,
		["heightHealth"] = 16,
		["heightPower"] = 4,
		showbuffs = false,
	},
	targettargettarget = {
		["initial-width"] = 175,
		["initial-height"] = 20,
		["heightHealth"] = 16,
		["heightPower"] = 4,
		showbuffs = false,
	},
	focus = {
		["initial-width"] = 200,
		["initial-height"] = 39,
		["heightHealth"] = 22,
		["heightPower"] = 16,
		powertext = true,
		showbuffs = true,		
	},
	focustarget = {
		["initial-width"] = 150,
		["initial-height"] = 39,
		["heightHealth"] = 22,
		["heightPower"] = 16,
		powertext = true,
		showbuffs = false,		
	},	
	pet = {
		["initial-width"] = 150,
		["initial-height"] = 39,	
		["heightHealth"] = 22,
		["heightPower"] = 16,
		powertext = true,
		showbuffs = false,		
	},
	pettarget = {
		["initial-width"] = 150,
		["initial-height"] = 39,
		["heightHealth"] = 22,
		["heightPower"] = 16,
		powertext = true,
		showbuffs = false,		
	},
	_default = {
		["initial-width"] = 200,
		["initial-height"] = 39,
		["heightHealth"] = 22,
		["heightPower"] = 16,
		buffs = false,
		powertext = false,
		showbuffs = false,		
	}
}, 
{__index = function(t, k) 
	return t["_default"] 
end
})

formats.player.health = fmt_full
formats.pet.health = fmt_full
formats.party.health = fmt_full

formats.target.health = fmt_percminmax
formats.target.power = fmt_percminmax

formats.focus.health = fmt_percminmax
formats.focus.power = fmt_percminmax

formats.partypet.health = fmt_deficit
formats.partytarget.health = fmt_perc

formats.targettarget.health = fmt_percminmax
formats.targettarget.power = fmt_perc

formats.focustarget.health = fmt_deficit
formats.focustarget.power = fmt_perc
	

local runeloadcolors = {
	[1] = {0.77, 0.12, 0.23},
	[2] = {0.77, 0.12, 0.23},
	[3] = {0.4, 0.8, 0.1},
	[4] = {0.4, 0.8, 0.1},
	[5] = {0, 0.4, 0.7},
	[6] = {0, 0.4, 0.7},
}

local colors = setmetatable({
	power = setmetatable({
		['MANA'] = {0, 144/255, 1},
	}, {__index = oUF.colors.power}),
	runes = setmetatable({
		[1] = {0.77, 0.12, 0.23},
		[2] = {0.3, 0.8, 0.1},
		[3] = {0, 0.4, 0.7},
		[4] = {0.8, 0.8, 0.8},
	}, {__index = oUF.colors.runes}),
}, {__index = oUF.colors})

local sbColors = {
	green = { 0, 1, 0 },
	gray = { 0.5, 0.5, 0.5 },
	white = { 1, 1, 1},
	unknown = { .41, .95, .2 },
}

local classificationFormats = {
	worldboss = '|cffD7BEA5Boss|r',
	rareelite = '|cffD7BEA5%s+ Rare|r',
	elite = '|cffD7BEA5%s+|r',
	rare = '|cffD7BEA5%s Rare|r',
	normal = '|cffD7BEA5%s|r',
	trivial = '|cffD7BEA5%s|r',
}


local getDifficultyColor = function(level)
	if tonumber(level) ~= nil then
		local levelDiff = level - UnitLevel("player")
		if levelDiff >= 5 then
			return 1.00, 0.10, 0.10
		elseif levelDiff >= 3 then
			return 1.00, 0.50,0.25
		elseif levelDiff >= -2 then
			return 1.00, 1.00, 1.00
		elseif -levelDiff <= GetQuestGreenRange() then
			return 0.25, 0.75, 0.25
		end
		return 0.50, 0.50, 0.50
	end
end

-- This is the core of RightClick menus on diffrent frames
local menu = function(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("^%l", string.upper)

	if(cunit == 'Vehicle') then
		cunit = 'Pet'
	end

	if(unit == "party" or unit == "partypet") then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif(_G[cunit.."FrameDropDown"]) then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end

-- Deathknight
local function UpdateRuneBar(self, elapsed)
	local start, duration, ready = GetRuneCooldown(self:GetID())

	if(ready) then
		self:SetValue(1)
		self:SetScript('OnUpdate', nil)
	else
		self:SetValue((GetTime() - start) / duration)
	end
end

local function UpdateRunePower(self, event, rune, usable)
	for i = 1, 6 do
		if(rune == i and not usable and GetRuneType(rune)) then
			self.RuneBar[i]:SetScript('OnUpdate', UpdateRuneBar)
		end
	end
end

local function UpdateRuneType(self, event, rune)
	if(rune) then
		local runetype = GetRuneType(rune)
		if(runetype) then
			self.RuneBar[rune]:SetStatusBarColor(unpack(self.colors.runes[runetype]))
		end
	else
		for i = 1, 6 do
			local runetype = GetRuneType(i)
			if(runetype) then
				self.RuneBar[i]:SetStatusBarColor(unpack(self.colors.runes[runetype]))
			end
		end
	end
end
-- Deathknight - End


local function updateName(self, event, unit)
	if(self.unit == unit) then
		self.Name:SetTextColor(sbColors.white[1], sbColors.white[2], sbColors.white[3])

		local uName = UnitName(unit)
				
		-- name unknow pets just "pet"
		if (unit == 'pet' and uName == 'Unknown') then 
			uName = 'Pet'
		end
		if self.Class then
			if UnitIsPlayer(unit)  then	
				self.Class:SetText(UnitClass(unit))
			else
				self.Class:SetText(UnitCreatureFamily(unit) or UnitCreatureType(unit))
			end		
		end
		if string.len(uName) > 20 then
			self.Name:SetFormattedText('%s...', uName:sub(1, 20))
		else
			self.Name:SetText(uName)
		end
		
		if(self.Lvl) then	
			local lvl = self.Lvl
			local level = UnitLevel(unit)
			local classification = UnitClassification(unit)

			if (level == -1) then
				level = "??"
			end
			
			if classificationFormats[classification] then
				lvl:SetFormattedText(classificationFormats[classification], level)
			else			
				lvl:SetText(level)
			end
			if UnitCanAttack("player", unit) then
				lvl:SetTextColor(getDifficultyColor(level))
			else
				lvl:SetTextColor(1, 1, 1)
			end
		end
	
		if self.Race then
			self.Race:SetText(UnitRace(unit))
		end	
	
	end
end

local function PostUpdateHealth(bar, unit, min, max)

	local cur, maxHealth = min, max
		
	if(max ~= 0) then
		r, g, b = oUF.ColorGradient(min/max, .69,.31,.31, .65,.63,.35, .33,.59,.33)
	end
	
	--local color = colors.class[UnitPowerType(unit)]

	if UnitIsDead(unit) then
		bar:SetValue(0)
		bar.value:SetText("Dead")
	elseif UnitIsGhost(unit) then
		bar:SetValue(0)
		bar.value:SetText("Ghost")	
	elseif not UnitIsConnected(unit) then
		bar:SetValue(0)
		bar.value:SetText("Offline")
	else
		formats[unit].health(bar.value, cur, maxHealth)
		--bar.value:SetText(cur.." / "..maxHealth.." - (|cffFFFFFF"..floor(cur/maxHealth*100).." %|r)")
	end
	
	local color
	
	if unit == "pet" then
		--local num, str = UnitPowerType(unit)
		color = colors.happiness[GetPetHappiness()] or colors.health --self.colors.power[UnitPowerType(unit)]
		--print(unit.." - "..UnitPowerType(unit))
	elseif UnitIsPlayer(unit) then
		color = colors.class[select(2, UnitClass(unit))]		
		--print(unit.." - "..color[1])
		--color[1], color[2], color[2] = self.ColorGradient(min/max, color[1], color[2], color[3], 0, 0, 0)
	elseif UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) and not UnitIsPlayer(unit) then
		--color = sbColors.gray		
		--@TODO: unpack needed?
		--color = unpack(self.colors.tapped)
		color = colors.tapped
	else
		color = colors.reaction[UnitReaction(unit, 'player')] or colors.health
	end	
	
	if not color then 
		color = sbColors.unknown
	end	
	
	if(UnitIsDead(unit) or UnitIsGhost(unit) or not UnitIsConnected(unit)) then
		--color = self.colors.disconnected
		bar:SetStatusBarColor(.31*.25, .45*.25, .63*.25)
		bar.bg:SetVertexColor(.31*.25, .45*.25, .63*.25)
	else
		bar:SetStatusBarColor(color[1], color[2], color[3])
		bar.bg:SetTexture(color[1], color[2], color[3], .25)		
	end
	
	return updateName(bar:GetParent(), 'PostUpdateHealth', unit)
	--self:UNIT_NAME_UPDATE(event, unit)
end

local PostUpdatePower = function(bar, unit, min, max)
--local function PostUpdatePower(self, event, unit, bar, min, max)

	if max == 0 or UnitIsDead(unit) or UnitIsGhost(unit) or not UnitIsConnected(unit) then
		bar:SetValue(0)
		if bar.value then
			bar.value:SetText()
		end
	elseif bar.value then
		formats[unit].power(bar.value, min, max)
		--bar.value:SetText(min.."/"..max)
	end

	--fix coloring issues TODO: CHECK RESULTS
	local num, str = UnitPowerType(unit)
	local color = colors.power[select(4, GetBuildInfo()) >= 3e4 and str or num]	
	--local color = colors.power[UnitPowerType(unit)]
	
	if not color then 
		color = sbColors.white
	end	
		
	if color then
		bar:SetStatusBarColor(color[1], color[2], color[3])
		bar.bg:SetTexture(color[1], color[2], color[3], .25)
	end
	
	--self:UNIT_NAME_UPDATE(event, unit)
end

local function PostUpdateExperience(self, event, unit, bar, min, max)
	if(GetXPExhaustion()) then
		bar:SetStatusBarColor(.65,.33,.73) -- rested color
	else
		bar:SetStatusBarColor(.33,.59,.33) -- normal color
	end
end

local auraIcon = function(self, button)
	button.icon:SetTexCoord(.07, .93, .07, .93)

	local tex = button:CreateTexture(nil, "OVERLAY")
	tex:SetTexture(bordertexture)
	tex:SetAllPoints(button)

	if button.overlay then -- hide debuff overlay from oUF and set our own
		button.overlay:Hide()
		button.overlay = tex
	else
		tex:SetVertexColor(.25,.25,.35)
	end
end

local function PostCreateAuraIcon(self, button, icons, index, debuff)
	icons.showDebuffType = true
	button.cd:SetReverse()
	button.icon:SetTexCoord(.07, .93, .07, .93)
	button.count:SetPoint('BOTTOMRIGHT', button, 1, 0)
	button.count:SetTextColor(.84,.75,.65)
	button:SetScript('OnMouseUp', function(self, mouseButton)
		if mouseButton == 'RightButton' then
			local name, rank = UnitBuff('player', index)
			CancelPlayerBuff(name, rank)
		end
	end)

	local tex = button:CreateTexture(nil, "OVERLAY")
	tex:SetTexture(bordertexture)
	tex:SetAllPoints(button)	
end



local OnEnter = function(self)
	if not InCombatLockdown() then
		UnitFrame_OnEnter(self)
	end
end

local OnLeave = function(self)
	if not InCombatLockdown() then
		UnitFrame_OnLeave()
	end
end

local RAID_TARGET_UPDATE = function(self, event)
	local index = GetRaidTargetIndex(self.unit)
	if(index) then
		self.RIcon:SetText(ICON_LIST[index].."22|t")
	else
		self.RIcon:SetText()
	end
end

--local func = function(settings, self, unit)
local function Shared(self, unit)
	self.menu = menu

	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)

	--self:RegisterForClicks"anyup"
	-- XXX: Change to AnyUp when RegisterAttributeDriver doesn't cause clicks
	self:RegisterForClicks('AnyDown')
	
	-- Backgrounds
	self:SetBackdrop(backdrop)
	self:SetBackdropColor(0,0,0,1)
	
	-- Frame size calculation
	local settings = UnitSettings[unit]
	
	local width = settings["initial-width"] or 220
	local height = settings["initial-height"] or 22
	local heightHealth = settings["heightHealth"] or 16
	local heightPower = settings["heightPower"] or 16
	
	local showBuffs = settings["showbuffs"]
	
	self:SetHeight(height)
	self:SetWidth(width)

	-- Healthbar
	local Health = CreateFrame("StatusBar", nil, self)
	--Health:SetHeight(tiny and 16 or 22) -- 16 height if we're dealing with tiny or micro frames
	Health:SetHeight(heightHealth)
	Health:SetStatusBarTexture(statusbartexture)
	Health:SetPoint("TOPLEFT")
	Health:SetPoint("TOPRIGHT")
	Health.frequentUpdates = true
	Health.Smooth = true	
	
	-- Healthbar background
	local HealthBG = Health:CreateTexture(nil, "BORDER")
	HealthBG:SetAllPoints(Health)
	HealthBG:SetTexture(.25,.25,.25)

	-- Healthbar text
	local HealthText = Health:CreateFontString(nil, "OVERLAY")
	HealthText:SetPoint("RIGHT", -2, -1)
	HealthText:SetFont(font, fontHeight)
	HealthText:SetShadowColor(0, 0, 0)
	HealthText:SetShadowOffset(0.8, -0.8)
	HealthText:SetTextColor(1,1,1)

	-- Call the functions
	Health.bg = HealthBG
	Health.value = HealthText
	self.Health = Health

	local name = Health:CreateFontString(nil, "OVERLAY")
	name:SetPoint("LEFT", 2, -1)
	--name:SetPoint("RIGHT", HealthText, "LEFT")
	name:SetJustifyH("LEFT")
	name:SetFont(font, fontHeight)
	name:SetShadowColor(0, 0, 0)
	name:SetShadowOffset(0.8, -0.8)
	name:SetTextColor(1,1,1)
	self.Name = name
	
	local Power, PowerBG, PowerText
	-- Power Bar 
	Power = CreateFrame("StatusBar", nil, self)
	
	
	Power:SetHeight(heightPower)
	Power:SetStatusBarTexture(statusbartexture)

	-- Power:SetParent(self)
	Power:SetPoint("LEFT")
	Power:SetPoint("RIGHT")
	Power:SetPoint("TOP", Health, "BOTTOM", 0, -1.35)
	
	PowerBG = Power:CreateTexture(nil, "BORDER")
	PowerBG:SetAllPoints(Power)
	PowerBG:SetTexture(.25,.25,.25)
	Power.bg = PowerBG
	
	--experimental
	Power.frequentUpdates = true
	Power.Smooth = true

	Power.colortapping = true
	Power.colorDisconnected = true
	Power.colorClass = true
	Power.colorReaction = true
	
	self.Power = Power
	
	if(unit == "pet") then
		self:RegisterEvent("UNIT_HAPPINESS", updateName)
	end
	
	if(unit == "player") then
		local state = Health:CreateTexture(nil, "OVERLAY")
		state:SetHeight(16)
		state:SetWidth(16)
		state:SetPoint("TOPLEFT", self, -13, -30)
		state:SetTexture("Interface\\CharacterFrame\\UI-StateIcon")
		state:SetTexCoord(0.57, 0.90, 0.08, 0.41)
		self.Combat = state
	
		if UnitLevel('player') ~= MAX_PLAYER_LEVEL then
			-- Resting icon
			local rest = self:CreateTexture(nil, "OVERLAY")
			rest:SetHeight(24)
			rest:SetWidth(24)
			rest:SetPoint("CENTER", self, "TOPLEFT", -5, -1)
			rest:SetTexture([[Interface\CharacterFrame\UI-StateIcon]])
			rest:SetTexCoord(0, 1/2, 0, 0.421875)
			self.Resting = rest
		end
		
		
		if(playerClass == 'DEATHKNIGHT') then
			self.RuneBar = {}
			for i = 1, 6 do
				
			
				self.RuneBar[i] = CreateFrame('StatusBar', nil, self)
				if(i == 1) then
					self.RuneBar[i]:SetPoint('TOPLEFT', self, 'BOTTOMLEFT', 0, -1)
				else
					self.RuneBar[i]:SetPoint('TOPLEFT', self.RuneBar[i-1], 'TOPRIGHT', 1, 0)
				end
				self.RuneBar[i]:SetStatusBarTexture(statusbartexture)
				self.RuneBar[i]:SetStatusBarColor(unpack(runeloadcolors[i]))
				self.RuneBar[i]:SetHeight(4)
				self.RuneBar[i]:SetWidth(200/6 - 0.85)
				self.RuneBar[i]:SetBackdrop(backdrop)
				self.RuneBar[i]:SetBackdropColor(0, 0, 0)
				self.RuneBar[i]:SetMinMaxValues(0, 1)
				self.RuneBar[i]:SetID(i)

				self.RuneBar[i].bg = self.RuneBar[i]:CreateTexture(nil, 'BORDER')
				self.RuneBar[i].bg:SetAllPoints(self.RuneBar[i])
				self.RuneBar[i].bg:SetTexture(0.3, 0.3, 0.3)			
			end		
			
			RuneFrame:Hide()

			self:RegisterEvent('RUNE_TYPE_UPDATE', UpdateRuneType)
			self:RegisterEvent('RUNE_REGEN_UPDATE', UpdateRuneType)
			self:RegisterEvent('RUNE_POWER_UPDATE', UpdateRunePower)			
			
		end
	end	
	
	if(unit == 'target') then
		local cpoints = self:CreateFontString(nil, "OVERLAY")
		cpoints:SetPoint("RIGHT", self, "LEFT", -9, 3)
		cpoints:SetFont(font, 38, "OUTLINE")
		cpoints:SetTextColor(1,1,1)
		cpoints:SetJustifyH("RIGHT")
		self:Tag(cpoints, '[cpoints]')
	end

	if (settings["powertext"]) then
	--if (unit == 'player' or unit == 'target'  or unit == 'focus' or unit == "focustarget" or unit == "party") then
		PowerText = Power:CreateFontString(nil, "OVERLAY")
		PowerText:SetPoint("RIGHT", -2, -1)
		PowerText:SetFont(font, fontHeight)
		PowerText:SetShadowColor(0, 0, 0)
		PowerText:SetShadowOffset(0.8, -0.8)	
		PowerText:SetTextColor(1,1,1)
		Power.value = PowerText	
	
		local lvl, class, race
		lvl = Power:CreateFontString(nil, "OVERLAY")
		lvl:SetPoint("LEFT", Power, "LEFT", 2, 0)
		lvl:SetFont(font, fontHeight)
		lvl:SetShadowColor(0, 0, 0)
		lvl:SetShadowOffset(0.8, -0.8)	
		self.Lvl = lvl

		class = Power:CreateFontString(nil, "OVERLAY")
		class:SetPoint("LEFT", lvl, "RIGHT",  1, 0)
		class:SetFont(font, fontHeight)
		class:SetShadowColor(0, 0, 0)
		class:SetShadowOffset(0.8, -0.8)
		self.Class = class
	
		race = Power:CreateFontString(nil, "OVERLAY")
		race:SetPoint("LEFT", class, "RIGHT",  1, 0)
		race:SetFont(font, fontHeight)
		race:SetTextColor(1,1,1)
		race:SetShadowColor(0, 0, 0)
		race:SetShadowOffset(0.8, -0.8)	
		self.Race = race	
	end
	
	local buffs	
	local buffheight = 12 -- which haPowerens to be the oUF default
	local buffwidth = width
	buffwidth = buffwidth - (buffwidth%16)
	
	if showBuffs then
		buffs = CreateFrame("Frame", nil, self)
		buffs.size = buffheight
		buffs:SetHeight(buffheight)
		buffs:SetWidth(buffwidth)

		local debuffs = CreateFrame("Frame", nil, self)
		debuffs.size = buffheight
		debuffs:SetHeight(buffheight)
		debuffs:SetWidth(buffwidth)
	
		debuffs:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT",1.5,3)
		debuffs.initialAnchor = "BOTTOMRIGHT"
		debuffs["growth-y"] = "UP"
		debuffs["growth-x"] = "LEFT"
		debuffs.num = 40
		self.Debuffs = debuffs
			
		buffs:SetPoint("TOPLEFT", self, "BOTTOMLEFT", -1.5, -8)
		buffs.initialAnchor = "TOPLEFT"
		buffs["growth-y"] = "DOWN"
		buffs.num = 40
		self.Buffs = buffs	
	end	
	
	if not unit or unit == "player" or unit == "pet" or unit:find("partypet%d") then
		self.DebuffHighlightBackdrop = true -- oUF_DebuffHighlight SuPowerort, using the backdrop
		self.DebuffHighlightFilter = true -- only show debuffs I can cure
	end
	
	local cbft = Health:CreateFontString(nil, "OVERLAY")
	cbft:SetPoint("CENTER", self, "CENTER")
	cbft:SetFont(font, 16, "OUTLINE")
	self.CombatFeedbackText = cbft		

	local Leader = self:CreateTexture(nil, "OVERLAY")
	Leader:SetHeight(16)
	Leader:SetWidth(16)
	Leader:SetPoint("TOPLEFT", self, 0, 8)	
	self.Leader = Leader
	
	local LFDRole = Health:CreateTexture(nil, "OVERLAY")
	LFDRole:SetHeight(16)
	LFDRole:SetWidth(16)
	LFDRole:SetPoint('LEFT', Leader, 'RIGHT', 18, 2)
	self.LFDRole = LFDRole		

	local MasterLooter = self:CreateTexture(nil, 'OVERLAY')
	MasterLooter:SetHeight(16)
	MasterLooter:SetWidth(16)
	MasterLooter:SetPoint('LEFT', Leader, 'RIGHT')
	self.MasterLooter = MasterLooter
	
	local PhaseIcon = self:CreateTexture(nil, 'OVERLAY')
	PhaseIcon:SetHeight(16)
	PhaseIcon:SetWidth(16)
	PhaseIcon:SetPoint('LEFT', Leader, 'RIGHT', 32, 2)
	self.PhaseIcon = PhaseIcon
	
	local pvp = Health:CreateTexture(nil, "OVERLAY")
	pvp:SetHeight(16)
	pvp:SetWidth(16)
	pvp:SetPoint("TOPLEFT", self, -12, -15)
	self.PvP = pvp
	
	local RaidIcon = Health:CreateFontString(nil, "OVERLAY")
	RaidIcon:SetPoint("TOP", self, 0, 8)
	RaidIcon:SetJustifyH"LEFT"
	RaidIcon:SetFontObject(GameFontNormalSmall)
	RaidIcon:SetTextColor(1, 1, 1)
	self.RIcon = RaidIcon
	self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
	table.insert(self.__elements, RAID_TARGET_UPDATE)	
	
	self.PostUpdateHealth = PostUpdateHealth
	self.PostUpdatePower = PostUpdatePower
	self.PostUpdateExperience = PostUpdateExperience
	self.PostCreateAuraIcon = PostCreateAuraIcon
	
	Health.PostUpdate = PostUpdateHealth
	Power.PostUpdate = PostUpdatePower

end

local func = function(settings, self, unit)
	if unit == "player" then
		if LibDruidMana then
			local druid = Health:CreateFontString(nil, "OVERLAY")
			druid:SetFontObject(GameFontNormalSmall)
			druid:SetTextColor(1,1,1)
			druid:SetPoint("CENTER", Power)
			druidmanabartexture = druid
		end
	end
end

oUF:RegisterStyle("sb", Shared)

oUF:Factory(function(self) 
	oUF:SetActiveStyle("sb")

	local player = self:Spawn("player")
	player:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 20, -70)

	local target = self:Spawn("target", "oUF_Target")
	target:SetPoint("LEFT", player, "RIGHT", 40, 0)

	local focus = oUF:Spawn("focus", "oUF_Focus")
	focus:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 20, -600)

	local party	= oUF:Spawn("header", "oUF_Party")
	
	local party = self:SpawnHeader(nil, nil, 'party,solo', 'showParty', true, 'showPlayer', false, 'yOffset', -31)	
	party:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 20, -150)
		
	local pet = oUF:Spawn("pet", "oUF_Pet")
	pet:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 20, -670)

	local targetofpet = oUF:Spawn("pettarget", "oUF_Pettarget")
	targetofpet:SetPoint("LEFT", pet, "RIGHT", 25, 0)

	local tof = oUF:Spawn("focustarget", "oUF_Focustarget")
	tof:SetPoint("LEFT", focus, "RIGHT", 25, 0)
	
	local tot = oUF:Spawn("targettarget", "oUF_TargetTarget")
	tot:SetPoint("TOPLEFT", target, "TOPRIGHT", 40, 0)

	local totot = oUF:Spawn("targettargettarget", "oUF_TargetTargetTarget")
	totot:SetPoint("TOP", tot, "BOTTOM", 0, -10)
	
	
	local bossFrame = {}
	for i = 1, MAX_BOSS_FRAMES, 1 do
		bossFrame[i] = oUF:Spawn("boss"..i, "oUF_Boss"..i)	
		if i == 1 then
		  bossFrame[i]:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 20, -740)
	   else
		  bossFrame[i]:SetPoint('TOP', bossFrame[i-1], 'BOTTOM', 0, -5)
	   end
	end

end)

-- The pet header is being a cunt, this is  better solution

local pets = {}
-- pets[1] = oUF:Spawn("partypet1", "oUF_PartyPet1")
-- pets[1]:SetPoint("TOPRIGHT", party, "TOPRIGHT", 0, -44)
-- for i =2, 4 do
	-- pets[i] = oUF:Spawn("partypet"..i, "oUF_PartyPet"..i)
	-- pets[i]:SetPoint("TOP", pets[i-1], "BOTTOM", 0, -50)
-- end

local pts = {}
-- pts[1] = oUF:Spawn("party1target", "oUF_Party1Target")
-- pts[1]:SetPoint("TOPLEFT", party, "TOPRIGHT", 5, 0)
-- for i =2, 4 do
	-- pts[i] = oUF:Spawn("party"..i.."target", "oUF_Party"..i.."Target")
	-- pts[i]:SetPoint("TOP", pts[i-1], "BOTTOM", 0, -50)
-- end

if LibDruidMana then
	LibDruidMana:AddListener(function(currMana, maxMana)
		if currMana ~= maxMana then
			local perc = floor(currMana/maxMana*100)
			druidmanabartexture:SetFormattedText("%d|cff0090ff%%|r", perc)
		else
			druidmanabartexture:SetText()
		end
	end)
end
